import { r as registerInstance, c as createEvent, h } from './index-ab63a587.js';
import { C as ControlInfoValue } from './utils-8de9c2f5.js';

const k2btEnhancedcomboCss = ":host{display:block}.K2BTEnhancedComboHeader{border-bottom:1px solid #ccc}.K2BTEnhancedComboHeader:after{content:'▼';width:0px}.K2BTEnhancedComboContentsOpen{display:-ms-flexbox;display:flex;-ms-flex-direction:column;flex-direction:column;position:absolute;border:1px solid #ccc;background-color:#fff;z-index:1}.K2BTEnhancedComboSearchInput{width:100%}.K2BTEnhancedComboItems{max-height:300px;overflow:auto;scrollbar-width:thin;scrollbar-color:#d7d7d7 transparent}.K2BTEnhancedComboItems::-webkit-scrollbar{width:6px}.K2BTEnhancedComboItems::-webkit-scrollbar-thumb{background-color:#d7d7d7;border:1px solid transparent;cursor:pointer}.K2BTEnhancedComboContentsClosed{display:none}.K2BTEnhancedComboItem{display:-ms-flexbox;display:flex;-ms-flex-direction:row;flex-direction:row;-ms-flex-align:center;align-items:center;padding:8px}.K2BTEnhancedComboItem:hover{background-color:#ddd;cursor:pointer}.K2BTEnhancedComboItem_TextContainer{display:-ms-flexbox;display:flex;-ms-flex-direction:column;flex-direction:column;-ms-flex-positive:1;flex-grow:1}.K2BTEnhancedComboSubtitle{font-size:12px}.K2BTEnhancedComboCategoryName{padding-left:5px}.K2BTEnchancedComboCategoryContents .K2BTEnhancedComboItem{padding-left:10px}.K2BTEnhancedComboIcon{width:30px;height:30px;-o-object-fit:cover;object-fit:cover;margin-right:4px}.K2BTEnhancedComboIconInvisible{visibility:hidden}.K2BTEnhancedComboNoItemsFound{width:100%;text-align:center;margin-top:40px;margin-bottom:40px}.K2BTEnhancedComboTrailingText{-ms-flex-positive:0;flex-grow:0}";

const K2btEnhancedcombo = class {
    constructor(hostRef) {
        registerInstance(this, hostRef);
        this.changeEvent = createEvent(this, "change", 7);
        this.searchChangedEvent = createEvent(this, "searchChanged", 7);
        this.newRecordClickedEvent = createEvent(this, "newRecordClicked", 7);
        this.value = null;
        this.includesearch = true;
        this.includeemptyitem = true;
        this.emptyitemtext = '(none)';
        this.noresultsfoundtext = 'No results found';
        this.enableadditem = true;
        this.additemcaption = 'New record';
        this.searchfieldplaceholder = 'Search';
        this.searchvalue = '';
        this.enabled = true;
        this.readonlyclass = '';
        this.container = null;
        this.selectedElement = null;
        this.searchfield = null;
        this._keyboardSearchPrefix = '';
        this._keyboardSearchLastKeystroke = null;
    }
    processKeydown(event) {
        if (event.key == 'Escape') {
            this.resetPrefixSearch();
            // escape key
            this.open = false;
            event.stopPropagation();
        }
        else if (event.key == 'Enter') {
            this.resetPrefixSearch();
            // enter key
            this.open = !this.open;
            event.stopPropagation();
            return false;
        }
        else if (event.key == 'ArrowDown') {
            this.resetPrefixSearch();
            event.stopPropagation();
            event.preventDefault();
            //down key
            var currentValues = ControlInfoValue.getAtomicValues_impl(this.getFilteredValues());
            if (currentValues.length > 0) {
                var pos = currentValues.findIndex(v => v.value.trim() == this.value.trim());
                if (pos == -1) {
                    this.setValueWithoutClosing(currentValues[0].value.trim());
                }
                else {
                    pos = (((pos + 1) % currentValues.length) + currentValues.length) % currentValues.length;
                    this.setValueWithoutClosing(currentValues[pos].value.trim());
                }
            }
        }
        else if (event.key == 'ArrowUp') {
            this.resetPrefixSearch();
            event.stopPropagation();
            event.preventDefault();
            //up key
            var currentValues = ControlInfoValue.getAtomicValues_impl(this.getFilteredValues());
            if (currentValues.length > 0) {
                var pos = currentValues.findIndex(v => v.value.trim() == this.value.trim());
                if (pos == -1) {
                    this.setValueWithoutClosing(currentValues[currentValues.length - 1].value.trim());
                }
                else {
                    pos = (((pos - 1) % currentValues.length) + currentValues.length) % currentValues.length;
                    this.setValueWithoutClosing(currentValues[pos].value.trim());
                }
            }
        }
        else {
            var key = event.key;
            // Check if the key pressed maps to a single character
            if (key.length == 1) {
                if (document.activeElement != this.searchfield) {
                    var currentDate = Date.now();
                    if (this._keyboardSearchLastKeystroke == null || currentDate - this._keyboardSearchLastKeystroke > 3000) {
                        this._keyboardSearchPrefix = '';
                    }
                    this._keyboardSearchPrefix = this._keyboardSearchPrefix + key;
                    var currentValues = ControlInfoValue.getAtomicValues_impl(this.getFilteredValues());
                    var selectedValue = currentValues.find(v => v.description.toLowerCase().indexOf(this._keyboardSearchPrefix.toLowerCase()) == 0);
                    if (selectedValue != null) {
                        this.setValueWithoutClosing(selectedValue.value.trim());
                    }
                    this._keyboardSearchLastKeystroke = currentDate;
                }
            }
        }
    }
    onFocusIn() {
        this.resetPrefixSearch();
    }
    resetPrefixSearch() {
        this._keyboardSearchLastKeystroke = null;
    }
    closeMenu(ev) {
        this.resetPrefixSearch();
        if (!this.container.contains(ev.target))
            this.open = false;
    }
    getHeaderContent() {
        if (this.value == null)
            return this.emptyitemtext;
        else
            var vals = this.getAtomicValues().filter(value => value.value.trim() == this.value.trim());
        if (vals.length > 0)
            return vals[0].description;
        else
            return this.emptyitemtext;
    }
    toggleContentVisibilty() {
        this.resetPrefixSearch();
        this.open = !this.open;
    }
    getRawValues() {
        var result;
        if (this.values == null) {
            result = [];
        }
        else if (typeof this.values === 'string') {
            if (this.values == '')
                result = [];
            else
                result = JSON.parse(this.values);
        }
        else {
            result = [...this.values];
        }
        if (this.includeemptyitem) {
            result.unshift({
                value: '',
                description: this.emptyitemtext,
                imageUrl: '',
                detail: '',
                trailingText: '',
                items: [],
            });
        }
        return result;
    }
    getFilteredValues() {
        var result = this.getRawValues();
        if (this.includesearch && this.searchvalue != '')
            result = this.getFilteredValues_impl(result);
        return result;
    }
    getFilteredValues_impl(origin) {
        var result = [];
        if (origin != undefined) {
            for (let cv of origin) {
                if (cv.description.toLocaleLowerCase().indexOf(this.searchvalue.toLocaleLowerCase()) != -1 ||
                    cv.detail.toLocaleLowerCase().indexOf(this.searchvalue.toLocaleLowerCase()) != -1 ||
                    cv.trailingText.toLocaleLowerCase().indexOf(this.searchvalue.toLocaleLowerCase()) != -1) {
                    result = result.concat(cv);
                }
                else {
                    // Check if child items must be added
                    var filteredChildren = this.getFilteredValues_impl(cv.items);
                    if (filteredChildren.length > 0) {
                        var cv2 = Object.assign({}, cv);
                        cv2.items = filteredChildren;
                        result = result.concat(cv2);
                    }
                }
            }
        }
        return result;
    }
    setValue(value) {
        this.setValueWithoutClosing(value);
        this.open = false;
    }
    setValueWithoutClosing(value) {
        this.value = value;
        this.changeEvent.emit(value);
    }
    changeSearchValue(value) {
        this.searchvalue = value;
        this.searchChangedEvent.emit({ value });
    }
    onIncludeNewRecordClick() {
        this.open = false;
        this.newRecordClickedEvent.emit(null);
    }
    render() {
        this.selectedElement = null;
        return (h("div", null, h("p", { class: "form-control-static", style: this.enabled ? { display: 'none' } : {} }, h("span", { class: this.readonlyclass }, this.getHeaderContent())), h("div", { tabindex: "0", class: "K2BTEnhancedCombo", ref: node => (this.container = node), style: !this.enabled ? { display: 'none' } : {} }, h("div", { class: "K2BTEnhancedComboHeader", onClick: () => this.toggleContentVisibilty() }, this.getHeaderContent()), h("div", { class: this.open ? 'K2BTEnhancedComboContentsOpen' : 'K2BTEnhancedComboContentsClosed' }, this.includesearch ? (h("input", { type: "text", class: "K2BTEnhancedComboSearchInput", ref: c => (this.searchfield = c), onInput: event => this.changeSearchValue(event.target.value), placeholder: this.searchfieldplaceholder })) : (''), h("div", { class: "K2BTEnhancedComboItems" }, this.getComboContent()), this.enableadditem ? (h("span", { class: "K2BTEnhancedComboNewAction", onClick: () => this.onIncludeNewRecordClick() }, this.additemcaption)) : ('')))));
    }
    getAtomicValues() {
        return ControlInfoValue.getAtomicValues_impl(this.getRawValues());
    }
    getComboContent() {
        const rawValues = ControlInfoValue.getAtomicValues_impl(this.getRawValues());
        var containsDetails = ControlInfoValue.containsDetails(rawValues);
        var containsIcons = ControlInfoValue.containsIcons(rawValues);
        var containsTrailingText = ControlInfoValue.containsTrailingText(rawValues);
        const filteredValues = !this.open ? [] : this.getFilteredValues();
        if (filteredValues.length > 0)
            return filteredValues.map(item => this.getItemContent(item, containsDetails, containsIcons, containsTrailingText));
        else
            return h("div", { class: "K2BTEnhancedComboNoItemsFound" }, this.noresultsfoundtext);
    }
    onImageError(e) {
        e.target.classList.add('K2BTEnhancedComboIconInvisible');
    }
    getItemContent(item, containsDetails, containsIcons, containsTrailingText) {
        var _a;
        if (((_a = item.items) === null || _a === void 0 ? void 0 : _a.length) > 0) {
            // Item is a category
            return (h("div", { class: "K2BTEnhancedComboCategory" }, h("span", { class: "K2BTEnhancedComboCategoryName" }, item.description), h("div", { class: "K2BTEnchancedComboCategoryContents" }, item.items.map(item => this.getItemContent(item, containsDetails, containsIcons, containsTrailingText)))));
        }
        else {
            return (h("div", { class: this.value.trim() == item.value.trim() ? 'K2BTEnhancedComboItem K2BTEnhancedComboSelected' : 'K2BTEnhancedComboItem', ref: c => {
                    if (this.value.trim() == item.value.trim())
                        this.selectedElement = c;
                }, onClick: () => this.setValue(item.value.trim()) }, containsIcons ? (h("div", { class: "K2BTEnhancedComboIconContainer" }, h("img", { class: "K2BTEnhancedComboIcon", src: item.imageUrl, onError: e => this.onImageError(e) }))) : (''), h("div", { class: "K2BTEnhancedComboItem_TextColumn" }, h("div", { class: "K2BTEnhancedComboItem_TextContainer" }, h("span", { class: "K2BTEnhancedComboDescription" }, this.getHighlightedText(item.description)), containsTrailingText ? h("span", { class: "K2BTEnhancedComboTrailingText" }, this.getHighlightedText(item.trailingText)) : ''), h("div", { class: "K2BTEnhancedComboItem_TextContainer" }, containsDetails ? h("span", { class: "K2BTEnhancedComboSubtitle" }, this.getHighlightedText(item.detail)) : ''))));
        }
    }
    getHighlightedText(originalText) {
        var position = originalText.toLowerCase().indexOf(this.searchvalue.toLowerCase());
        if (this.searchvalue == undefined || this.searchvalue == '' || position == -1) {
            return h("span", null, originalText);
        }
        else {
            return (h("span", null, originalText.substring(0, position), h("span", { class: "K2BTEnhancedComboSearchHighlight" }, originalText.substring(position, position + this.searchvalue.length)), this.getHighlightedText(originalText.substring(position + this.searchvalue.length))));
        }
    }
    componentDidRender() {
        if (this.selectedElement != null) {
            this.selectedElement.scrollIntoView({ block: 'nearest' });
        }
    }
    componentDidLoad() {
        var currentValues = this.getAtomicValues();
        if (!this.includeemptyitem && currentValues.filter(v => v.value.trim() == this.value.trim()).length == 0) {
            if (currentValues.length > 0) {
                this.setValueWithoutClosing(currentValues[0].value.trim());
            }
        }
    }
};
K2btEnhancedcombo.style = k2btEnhancedcomboCss;

export { K2btEnhancedcombo as k2bt_enhancedcombo };
