import { r as registerInstance, c as createEvent, h } from './index-ab63a587.js';
import { C as ControlInfoValue } from './utils-8de9c2f5.js';

const k2btEnhancedsuggestCss = ":host{display:block}.K2BTEnhancedCombo{float:left}.K2BTEnhancedComboHeader{border-bottom:1px solid #ccc}.K2BTEnhancedComboHeader:after{content:'▼';width:0px}.K2BTEnhancedComboContentsOpen{display:-ms-flexbox;display:flex;-ms-flex-direction:column;flex-direction:column;position:absolute;border:1px solid #ccc;background-color:#fff;z-index:1}.K2BTEnhancedComboSearchInput{width:100%}.K2BTEnhancedComboItems{max-height:300px;overflow:auto;scrollbar-width:thin;scrollbar-color:#d7d7d7 transparent}.K2BTEnhancedComboItems::-webkit-scrollbar{width:6px}.K2BTEnhancedComboItems::-webkit-scrollbar-thumb{background-color:#d7d7d7;border:1px solid transparent;cursor:pointer}.K2BTEnhancedComboContentsClosed{display:none}.K2BTEnhancedComboItem{display:-ms-flexbox;display:flex;-ms-flex-direction:row;flex-direction:row;-ms-flex-align:center;align-items:center;padding:8px}.K2BTEnhancedComboItem:hover{background-color:#ddd;cursor:pointer}.K2BTEnhancedComboItem_TextContainer{display:-ms-flexbox;display:flex;-ms-flex-direction:column;flex-direction:column;-ms-flex-positive:1;flex-grow:1}.K2BTEnhancedComboSubtitle{font-size:12px}.K2BTEnhancedComboCategoryName{padding-left:5px}.K2BTEnchancedComboCategoryContents .K2BTEnhancedComboItem{padding-left:10px}.K2BTEnhancedComboIcon{width:30px;height:30px;-o-object-fit:cover;object-fit:cover;margin-right:4px}.K2BTEnhancedComboIconInvisible{visibility:hidden}.K2BTEnhancedComboNoItemsFound{width:100%;text-align:center;margin-top:40px;margin-bottom:40px}.K2BTEnhancedComboTrailingText{-ms-flex-positive:0;flex-grow:0}";

const K2btEnhancedsuggest = class {
    constructor(hostRef) {
        registerInstance(this, hostRef);
        this.changeEvent = createEvent(this, "change", 7);
        this.newRecordClickedEvent = createEvent(this, "newRecordClicked", 7);
        this.inputValueChangedEvent = createEvent(this, "inputValueChanged", 7);
        this.value = null;
        this.noresultsfoundtext = 'No results found';
        this.enableadditem = true;
        this.additemcaption = 'New record';
        this.enabled = true;
        this.readonlyclass = '';
        this.searchvalue = '';
        this.placeholder = '';
        this.container = null;
        this.selectedElement = null;
    }
    processKeydown(event) {
        if (event.which == 27) {
            // escape key
            this.open = false;
            event.stopPropagation();
        }
        if (event.which == 13) {
            // enter key
            this.open = !this.open;
            this.searchvalue = this.getValueDescription();
            event.stopPropagation();
            return false;
        }
        if (this.open) {
            if (event.which == 40) {
                event.stopPropagation();
                event.preventDefault();
                //down key
                var currentValues = ControlInfoValue.getAtomicValues_impl(this.getFilteredValues());
                if (currentValues.length > 0) {
                    var pos = currentValues.findIndex(v => v.value == this.value);
                    if (pos == -1) {
                        this.setValueWithoutClosing(currentValues[0].value);
                    }
                    else {
                        pos = (((pos + 1) % currentValues.length) + currentValues.length) % currentValues.length;
                        this.setValueWithoutClosing(currentValues[pos].value);
                    }
                }
            }
            else if (event.which == 38) {
                event.stopPropagation();
                event.preventDefault();
                //up key
                var currentValues = ControlInfoValue.getAtomicValues_impl(this.getFilteredValues());
                if (currentValues.length > 0) {
                    var pos = currentValues.findIndex(v => v.value == this.value);
                    if (pos == -1) {
                        this.setValueWithoutClosing(currentValues[currentValues.length - 1].value);
                    }
                    else {
                        pos = (((pos - 1) % currentValues.length) + currentValues.length) % currentValues.length;
                        this.setValueWithoutClosing(currentValues[pos].value);
                    }
                }
            }
        }
    }
    closeMenu(ev) {
        if (!this.container.contains(ev.target))
            this.open = false;
    }
    setValue(value) {
        this.setValueWithoutClosing(value);
        this.searchvalue = this.getValueDescription();
        this.open = false;
    }
    setValueWithoutClosing(value) {
        this.value = value;
        this.changeEvent.emit(value);
    }
    getRawValues() {
        var result;
        if (this.values == null) {
            result = [];
        }
        else if (typeof this.values === 'string') {
            if (this.values == '')
                result = [];
            else
                result = JSON.parse(this.values);
        }
        else {
            result = [...this.values];
        }
        return result;
    }
    getFilteredValues() {
        if (!this.open)
            return [];
        var result = this.getRawValues();
        if (this.searchvalue != '')
            result = this.getFilteredValues_impl(result);
        return result;
    }
    getFilteredValues_impl(origin) {
        var result = [];
        if (origin != undefined) {
            for (let cv of origin) {
                if (cv.description.toLocaleLowerCase().indexOf(this.searchvalue.toLocaleLowerCase()) != -1) {
                    result = result.concat(cv);
                }
                else {
                    // Check if child items must be added
                    var filteredChildren = this.getFilteredValues_impl(cv.items);
                    if (filteredChildren.length > 0) {
                        var cv2 = Object.assign({}, cv);
                        cv2.items = filteredChildren;
                        result = result.concat(cv2);
                    }
                }
            }
        }
        return result;
    }
    onIncludeNewRecordClick() {
        this.open = false;
        this.newRecordClickedEvent.emit(null);
    }
    getAtomicValues() {
        return ControlInfoValue.getAtomicValues_impl(this.getRawValues());
    }
    getSuggestPopoverContent() {
        const rawValues = this.getRawValues();
        var containsDetails = ControlInfoValue.containsDetails(rawValues);
        var containsIcons = ControlInfoValue.containsIcons(rawValues);
        var containsTrailingText = ControlInfoValue.containsTrailingText(rawValues);
        const filteredValues = this.getFilteredValues();
        if (filteredValues.length > 0)
            return filteredValues.map(item => this.getItemContent(item, containsDetails, containsIcons, containsTrailingText));
        else
            return h("div", { class: "K2BTEnhancedComboNoItemsFound" }, this.noresultsfoundtext);
    }
    onImageError(e) {
        // TODO: Think how to handle this case
        e.target.classList.add('K2BTEnhancedComboIconInvisible');
    }
    getItemContent(item, containsDetails, containsIcons, containsTrailingText) {
        var _a;
        if (((_a = item.items) === null || _a === void 0 ? void 0 : _a.length) > 0) {
            // Item is a category
            return (h("div", { class: "K2BTEnhancedComboCategory" }, h("span", { class: "K2BTEnhancedComboCategoryName" }, item.description), h("div", { class: "K2BTEnchancedComboCategoryContents" }, item.items.map(item => this.getItemContent(item, containsDetails, containsIcons, containsTrailingText)))));
        }
        else {
            return (h("div", { class: this.value == item.value ? 'K2BTEnhancedComboItem K2BTEnhancedComboSelected' : 'K2BTEnhancedComboItem', ref: c => {
                    if (this.value == item.value)
                        this.selectedElement = c;
                }, onClick: () => this.setValue(item.value) }, containsIcons ? (h("div", { class: "K2BTEnhancedComboIconContainer" }, h("img", { class: "K2BTEnhancedComboIcon", src: item.imageUrl, onError: e => this.onImageError(e) }))) : (''), h("div", { class: "K2BTEnhancedComboItem_TextColumn" }, h("div", { class: "K2BTEnhancedComboItem_TextContainer" }, h("span", { class: "K2BTEnhancedComboDescription" }, this.getHighlightedText(item.description)), containsTrailingText ? h("span", { class: "K2BTEnhancedComboTrailingText" }, this.getHighlightedText(item.trailingText)) : ''), h("div", { class: "K2BTEnhancedComboItem_TextContainer" }, containsDetails ? h("span", { class: "K2BTEnhancedComboSubtitle" }, this.getHighlightedText(item.detail)) : ''))));
        }
    }
    getHighlightedText(originalText) {
        var position = originalText.toLowerCase().indexOf(this.searchvalue.toLowerCase());
        if (this.searchvalue == undefined || this.searchvalue == '' || position == -1) {
            return h("span", null, originalText);
        }
        else {
            return (h("span", null, originalText.substring(0, position), h("span", { class: "K2BTEnhancedComboSearchHighlight" }, originalText.substring(position, position + this.searchvalue.length)), this.getHighlightedText(originalText.substring(position + this.searchvalue.length))));
        }
    }
    changeSearchValue(value) {
        this.searchvalue = value;
        this.inputValueChangedEvent.emit(null);
    }
    getValueDescription() {
        var values = this.getAtomicValues();
        var v = values.find(val => val.value == this.value);
        if (v != undefined) {
            return v.description;
        }
        return this.value;
    }
    render() {
        this.selectedElement = null;
        return (h("div", null, h("span", { class: this.readonlyclass, style: this.enabled ? { display: 'none' } : {} }, this.getValueDescription()), h("div", { tabindex: "0", class: "K2BTEnhancedCombo", ref: node => (this.container = node), style: !this.enabled ? { display: 'none' } : {} }, h("input", { type: "text", class: "form-control Attribute_Trn K2BTEnhancedSuggestInput", value: this.searchvalue, onInput: event => this.changeSearchValue(event.target.value), placeholder: this.placeholder, onClick: () => (this.open = true) }), h("div", { class: this.open ? 'K2BTEnhancedComboContentsOpen' : 'K2BTEnhancedComboContentsClosed' }, h("div", { class: "K2BTEnhancedComboItems" }, this.getSuggestPopoverContent()), this.enableadditem ? (h("span", { class: "K2BTEnhancedComboNewAction", onClick: () => this.onIncludeNewRecordClick() }, this.additemcaption)) : ('')))));
    }
};
K2btEnhancedsuggest.style = k2btEnhancedsuggestCss;

export { K2btEnhancedsuggest as k2bt_enhancedsuggest };
